# -*- coding: utf-8 -*-
from plone.theme.interfaces import IDefaultPloneLayer
from webcouturier.dropdownmenu import msg_fact as _
from zope import schema
from zope.interface import Interface


class IDropdownConfiguration(Interface):
    """This interface defines the configlet for dropdown menus."""

    dropdown_depth = schema.Int(
        title=_(u'label_dropdown_depth', default=u'Depth of dropdown menus'),
        description=_(u'help_dropdown_depth',
                      default=u'How many levels to list after the top level.'),
        required=True,
        default=3
    )

    enable_caching = schema.Bool(
        title=_(u'label_enable_caching', default=u'Enable caching'),
        description=_(
            u'help_enable_caching',
            default=(u'WARNING! This is an experimental feature. '
                     u'This is using RAM to store cached template for '
                     u'dropdown menus. Technically every user and '
                     u'every visited section gets its own instance '
                     u"in the ram.cache. Don't enable this if you don't "
                     u'know what this is about. Disable this option '
                     u'if you get unexpected behavior of your global tabs.')),
        default=False,
        required=False
    )

    enable_parent_clickable = schema.Bool(
        title=_(u'label_enable_parent_clickable',
                default=u'Enable clicking menu items that have children'),
        description=_(
            u'help_enable_parent_clickable',
            default=(u'With this option enabled, every menu item is '
                     u'clickable. With this option disabled, an item is only '
                     u'clickable when it is not a parent so it has no '
                     u'children.')),
        default=True,
        required=False
    )

    enable_desc = schema.Bool(
        title=_(u'label_enable_desc',
                default=u'Show Description'),
        description=_(
            u'help_desc',
            default=(u'With this option enabled, description is also shown.')),
        default=False,
        required=False
    )

    enable_images = schema.Bool(
        title=_(u'label_enable_img',
                default=u'Show Images'),
        description=_(
            u'help_img',
            default=(u'With this option enabled, images are shown.')),
        default=False,
        required=False
    )

    img_size = schema.Choice(
        title=_(u'label_size', default=u'Size'),
        description=_(u'description_size', default=u'The size of the image'),
        default='icon',
        vocabulary='plone.app.vocabularies.ImagesScales',
    )


class IDropdownSpecific(IDefaultPloneLayer):
    """Marker interface that defines a Zope 3 browser layer.
    """


class IDropdownMenuViewlet(Interface):
    """ Marker interface.
        Implements new functionality to global navigation that lets you to
        have dropdown menus for global navigation tabs. Dropdown menu is
        builded with navigation portlet's policy, so dropdowns contain items
        that are only allowed for navigation portlet. If the item is disabled
        for navigation portlet, it is disabled for dropdown menu automatically
    """

    def getTabObject(tabUrl=''):
        """Get the submenu tree for tab object"""



# -*- coding: utf-8 -*-
from plone import api
from plone.app.testing import logout
from plone.registry.interfaces import IRegistry
from webcouturier.dropdownmenu.browser.interfaces import IDropdownConfiguration
from webcouturier.dropdownmenu.testing import DROPDOWN_INTEGRATION_TESTING
from zope.component import getUtility

import unittest


class ControlPanelTestCase(unittest.TestCase):

    layer = DROPDOWN_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.controlpanel = self.portal['portal_controlpanel']

    def test_controlpanel_has_view(self):
        request = self.layer['request']
        view = api.content.get_view(
            u'dropdown-controlpanel',
            self.portal,
            request
        )
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_is_protected(self):
        from AccessControl import Unauthorized
        logout()
        with self.assertRaises(Unauthorized):
            self.portal.restrictedTraverse('@@dropdown-controlpanel')

    def test_controlpanel_installed(self):
        actions = [a.getAction(self)['id']
                   for a in self.controlpanel.listActions()]
        self.assertIn('DropdownConfiguration', actions)

    def test_controlpanel_removed_on_uninstall(self):
        setup_tool = self.portal['portal_setup']
        with api.env.adopt_roles(['Manager']):
            setup_tool.runAllImportStepsFromProfile(
                'profile-webcouturier.dropdownmenu:uninstall'
            )
        actions = [a.getAction(self)['id']
                   for a in self.controlpanel.listActions()]
        self.assertNotIn('DropdownConfiguration', actions)


class RegistryTestCase(unittest.TestCase):

    layer = DROPDOWN_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.registry = getUtility(IRegistry)
        self.settings = self.registry.forInterface(IDropdownConfiguration)  # noqa: P001

    def test_css_class_blacklist_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'dropdown_depth'))
        self.assertEqual(self.settings.dropdown_depth, 3)

    def test_records_removed_on_uninstall(self):
        setup_tool = self.portal['portal_setup']
        with api.env.adopt_roles(['Manager']):
            setup_tool.runAllImportStepsFromProfile(
                'profile-webcouturier.dropdownmenu:uninstall'
            )

        records = [
            IDropdownConfiguration.__identifier__ + '.dropdown_depth',
        ]

        for r in records:
            self.assertNotIn(r, self.registry)



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.testing import z2
from Products.CMFCore.utils import getToolByName

import webcouturier.dropdownmenu


class WebcouturierDropdownmenuLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(package=webcouturier.dropdownmenu)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'webcouturier.dropdownmenu:default')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)
        workflowTool = getToolByName(portal, 'portal_workflow')   # noqa: P001
        workflowTool.setDefaultChain('simple_publication_workflow')
        for i in range(2):
            folder_id = 'folder-%s' % i
            portal.invokeFactory('Folder', folder_id)  # noqa: P001
        setRoles(portal, TEST_USER_ID, ['Member'])


DROPDOWN_FIXTURE = WebcouturierDropdownmenuLayer()


DROPDOWN_INTEGRATION_TESTING = IntegrationTesting(
    bases=(DROPDOWN_FIXTURE,),
    name='WebcouturierDropdownmenuLayer:IntegrationTesting'
)


DROPDOWN_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(DROPDOWN_FIXTURE,),
    name='WebcouturierDropdownmenuLayer:FunctionalTesting'
)


DROPDOWN_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        DROPDOWN_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE
    ),
    name='WebcouturierDropdownmenuLayer:AcceptanceTesting'
)



# -*- coding: utf-8 -*-
from AccessControl import Unauthorized
from Acquisition import aq_inner
from plone.app.layout.navigation.interfaces import INavtreeStrategy
from plone.app.layout.navigation.navtree import buildFolderTree
from plone.app.layout.navigation.root import getNavigationRoot
from plone.app.layout.viewlets import common
from plone.app.portlets.portlets.navigation import Assignment
from plone.memoize.compress import xhtml_compress
from plone.memoize.ram import cache
from Products.CMFCore.interfaces import IContentish
from Products.CMFPlone.browser.navtree import NavtreeQueryBuilder
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from webcouturier.dropdownmenu.browser.interfaces import IDropdownConfiguration
from webcouturier.dropdownmenu.browser.interfaces import IDropdownMenuViewlet
from zope.component import getMultiAdapter
from zope.interface import implementer

import plone.api


def _get_cfg(name):
    return plone.api.portal.get_registry_record(
        name,
        interface=IDropdownConfiguration
    )


class DropdownQueryBuilder(NavtreeQueryBuilder):
    """Build a folder tree query suitable for a dropdownmenu
    """

    def __init__(self, context):
        NavtreeQueryBuilder.__init__(self, context)
        self.query['path'] = {
            'query': '/'.join(context.getPhysicalPath()),
            'navtree_start': 1,
            'depth': _get_cfg('dropdown_depth')
        }


@implementer(IDropdownMenuViewlet)
class DropdownMenuViewlet(common.GlobalSectionsViewlet):
    """A custom version of the global navigation class that has to have
       dropdown menus for global navigation tabs objects
    """

    #
    # Define a cache key: every instance (probabily only one per site,
    # language and user gets its/his own cache so we don't get the
    # menu retrieved in the wrong language nor conflicts due to view
    # permissions. (A manager might be able to view more pages than an
    # anonymous user.)
    #
    def _render_cachekey(fun, self):
        return ''.join((
            self.selected_portal_tab,
            plone.api.portal.get_current_language(aq_inner(self.context)),
            str(plone.api.user.is_anonymous()),
        ))

    # Cache by

    # ---> Viewlet Name -> should be always identical since no one
    #                      will use two drop down menus, won't he?
    # ---> Selected/Higlighted Tab -> solved current problem
    # ---> By language -> expect to be fixed, if we cache by user
    # ---> User Name -> this is the worst part however if there are
    #                   many logged in users

    #
    # Summary: every user and every visited section gets its own
    # instance in the ram.cache.
    #
    # Suggestion: we should improve this in a manner that not every
    # user needs its own set of instances in the cache
    #
    # If we can't solve this issue caching won't make much use but
    # consume lots of ram.
    #
    # :-(

    #
    # Original template goes here
    #
    _template = ViewPageTemplateFile('dropdown_sections.pt')

    #
    # use cache decoration in order to store/retrieve function output
    # to/from cache
    @cache(_render_cachekey)
    def cached_viewlet(self):
        return xhtml_compress(self._template())

    def index(self):
        if self.enable_caching:
            return self.cached_viewlet()
        else:
            return self._template()

    recurse = ViewPageTemplateFile('dropdown_recurse.pt')

    @property
    def enable_caching(self):
        return _get_cfg('enable_caching')

    @property
    def enable_parent_clickable(self):
        return _get_cfg('enable_parent_clickable')

    @property
    def enableImages(self):
        return _get_cfg('enable_images')

    @property
    def imgSize(self):
        return _get_cfg('img_size')

    @property
    def enableDesc(self):
        return _get_cfg('enable_desc')

    def update(self):
        common.ViewletBase.update(self)  # Get portal_state and portal_url
        super(DropdownMenuViewlet, self).update()
        context = aq_inner(self.context)
        portal_props = plone.api.portal.get_tool('portal_properties')
        self.properties = portal_props.navtree_properties
        self.navroot_path = getNavigationRoot(context)
        self.navroot_path = getNavigationRoot(context)
        try:
            if IContentish.providedBy(context):
                uid = plone.api.content.get_uuid(obj=context)
            else:
                uid = None
        except TypeError:
            # ArcheTypes (such as PFG FormFolder) have no UID yet when the
            # factory gets called.
            uid = None
        self.data = Assignment(root_uid=uid)

    def getTabObject(self, tabUrl='', tabPath=None):
        if tabUrl == self.portal_state.navigation_root_url():
            # We are at the navigation root
            return ''
        if tabPath is None:
            # get path for current tab's object
            try:
                # we are in Plone > 3.0.x
                tabPath = tabUrl.split(self.site_url)[-1]
            except AttributeError:
                # we are in Plone 3.0.x world
                tabPath = tabUrl.split(self.portal_url)[-1]

            if tabPath == '' or '/view' in tabPath:
                # It's either the 'Home' or Image tab. It can't have
                # any dropdown.
                return ''

            if tabPath.startswith('/'):
                tabPath = tabPath[1:]
            elif tabPath.endswith('/'):
                # we need a real path, without a slash that might appear
                # at the end of the path occasionally
                tabPath = str(tabPath.split('/')[0])

            if '%20' in tabPath:
                # we have the space in object's ID that has to be
                # converted to the real spaces
                tabPath = tabPath.replace('%20', ' ').strip()

        if tabPath == '':
            return ''

        portal = self.portal_state.portal()
        portal_path = '/'.join(portal.getPhysicalPath())
        # Check to see if we are using a navigation root. If we are
        # virtual hosting the navigation root as its own domain,
        # the tabPath is no longer rooted at the main portal.
        try:
            if portal_path != self.navroot_path:
                portal = portal.restrictedTraverse(self.navroot_path)
            tabObj = portal.restrictedTraverse(tabPath, None)
        except Unauthorized:
            # Unauthorized can happen when error page is rendered in
            # unallowed context
            return ''

        if tabObj is None:
            # just in case we have missed any possible path
            # in conditions above
            return ''

        strategy = getMultiAdapter((tabObj, self.data), INavtreeStrategy)
        queryBuilder = DropdownQueryBuilder(tabObj)
        query = queryBuilder()

        data = buildFolderTree(tabObj, obj=tabObj, query=query,
                               strategy=strategy)

        bottomLevel = self.data.bottomLevel or self.properties.getProperty(
            'bottomLevel', 0)

        return self.recurse(children=data.get('children', []), level=1,
                            bottomLevel=bottomLevel).strip()



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from webcouturier.dropdownmenu.tests import layer

import unittest2 as unittest


class Helper(unittest.TestCase):

    @property
    def portal(self):
        return self.layer['portal']

    def setRoles(self, roles):
        setRoles(self.portal, TEST_USER_ID, roles)


class DropdownmenuTestCase(Helper):

    layer = layer.DROPDOWNMENU_INTEGRATION



# -*- coding: utf-8 -*-
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from webcouturier.dropdownmenu.testing import DROPDOWN_INTEGRATION_TESTING

import unittest2 as unittest
import zope.interface


class TestDropdownmenu(unittest.TestCase):

    layer = DROPDOWN_INTEGRATION_TESTING

    def setUp(self):
        portal = self.layer['portal']
        request = self.layer['request']
        from webcouturier.dropdownmenu.browser.dropdown import DropdownMenuViewlet  # noqa
        viewlet = DropdownMenuViewlet(portal, request, None, None)
        setRoles(portal, TEST_USER_ID, ['Manager'])

        # we have 2 folders created on the layer right away
        self.root_folders_ids = ['folder-0', 'folder-1']

        # update the dropdownmenu viewlet
        viewlet.update()

        for folder_id in self.root_folders_ids:
            self.assertIn(
                folder_id,
                [tab['id'] for tab in viewlet.portal_tabs]
            )

        setRoles(portal, TEST_USER_ID, ['Member'])
        self.portal = portal
        self.viewlet = viewlet

    def addSubFolders(self):
        # add some subfolders to one of the folders
        rf = getattr(self.portal, 'folder-0')
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        for i in range(2):
            rf.invokeFactory('Folder', 'sub-%s' % i)  # noqa: P001
        setRoles(self.portal, TEST_USER_ID, ['Member'])

        return rf.absolute_url()

    def test_no_subfolders_without_content(self):
        # since we don't have subfolders yet, we should not have dropdowns
        for tab_url in [getattr(self.portal, folder_id).absolute_url()
                        for folder_id in self.root_folders_ids]:
            self.assertEqual(self.viewlet.getTabObject(tab_url), '')

    def test_dropdownmenus_available(self):
        rf_url = self.addSubFolders()
        self.assertNotEqual(
            self.viewlet.getTabObject(rf_url),
            '',
            'We don\'t have the sub-folders available in the global navigation'
        )

    def test_subfolders_in_dropdownmenus(self):
        rf_url = self.addSubFolders()
        self.viewlet.update()
        self.assertIn(
            'href="http://nohost/plone/folder-0/sub-0"',
            self.viewlet.getTabObject(rf_url),
            'The sub-folder\'s URL is not available in the global navigation'
        )

    def test_leaks_in_dropdownmenus(self):
        rf_url = self.addSubFolders()
        self.viewlet.update()
        self.assertNotIn(
            'href="http://nohost/plone/folder-0"',
            self.viewlet.getTabObject(rf_url),
            'We have the leakage of the top level folders in the dropdownmenus'
        )


class TestINavigationRootDropdownmenu(unittest.TestCase):
    """ Test that the dropdownmenus play nice with different INavigationRoot.
    """

    layer = DROPDOWN_INTEGRATION_TESTING

    def setUp(self):
        portal = self.layer['portal']
        request = self.layer['request']
        # we should have 2 folders in the site's root from the layer. Lets
        # mark one of it's sub-folders ('sub-0') as a navigation root
        self.f1 = portal['folder-0']
        zope.interface.alsoProvides(self.f1, INavigationRoot)
        from webcouturier.dropdownmenu.browser.dropdown import DropdownMenuViewlet  # noqa
        viewlet = DropdownMenuViewlet(portal, request, None, None)
        viewlet.update()

        self.root_folders_ids = ['sub-0', 'sub-1']
        self.subfolders = ['sub-sub-0', 'sub-sub-1']
        self.portal = portal
        self.request = request
        self.viewlet = viewlet
        self.rf_url = self.f1.absolute_url()

    def test_no_root_folder(self):
        self.assertNotIn(
            '<a href="http://nohost/plone/folder-0"',
            self.viewlet.getTabObject(self.rf_url),
            'The root folder itself is in the globalnavigation'
        )

    def test_dropdownmenus_content(self):
        # Tests the tree builder to aply nice with the INavigationRoot
        self.assertNotIn(
            '<a href="http://nohost/plone/folder-1"',
            self.viewlet.getTabObject(self.rf_url),
            'The dropdown menus don\'t respect the iNavigationRoot.'
        )



# -*- coding: utf-8 -*-
from plone.testing import layered
from webcouturier.dropdownmenu.testing import DROPDOWN_FUNCTIONAL_TESTING

import doctest
import unittest2 as unittest


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests(
        [
            layered(
                doctest.DocFileSuite(
                    'browser.rst',
                ),
                layer=DROPDOWN_FUNCTIONAL_TESTING
            ),
        ]
    )
    return suite



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

version = '3.0.0'

setup(
    name='webcouturier.dropdownmenu',
    version=version,
    description='Dropdown menus for global navigation in Plone',
    long_description=(open('README.rst').read() + '\n' +
                      open('CHANGES.rst').read()),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Plone :: 5.0',
        'Framework :: Plone :: 5.1',
        'Framework :: Plone',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='web couturier dropdown menu navigation',
    author='Denys Mishunov',
    author_email='denys.mishunov@gmail.com',
    url='http://plone.org/products/webcouturier-dropdownmenu',
    license='GPL',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['webcouturier'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'AccessControl',
        'Acquisition',
        'plone.api',
        'plone.app.controlpanel',
        'plone.app.layout',
        'plone.app.portlets',
        'plone.browserlayer',
        'plone.memoize',
        'plone.theme',
        'Products.CMFCore',
        'Products.CMFPlone>=5.0',
        'Products.GenericSetup',
        'setuptools',
        'zope.component',
        'zope.formlib',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.schema',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
            'plone.testing>=5.0.0',
            'plone.app.contenttypes',
            'plone.app.robotframework[debug]',
        ],
    },
    entry_points="""
        [z3c.autoinclude.plugin]
        target = plone
    """
)



# -*- coding: utf-8 -*-
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from Products.CMFPlone import PloneMessageFactory as _
from webcouturier.dropdownmenu.browser.interfaces import IDropdownConfiguration


class DropdownControlPanelForm(RegistryEditForm):

    id = 'DropdownControlPanel'
    schema = IDropdownConfiguration
    schema_prefix = (
        'webcouturier.dropdownmenu.browser.interfaces.IDropdownConfiguration'
    )

    label = _(
        u'A dropdown menu configuration.',
        default=u'A dropdown menu configuration.'
    )
    description = _(
        u'Settings to configure dropdown menus for global navigation.',
        default=u'Settings to configure dropdown menus for global navigation.'
    )


class DropdownControlPanel(ControlPanelFormWrapper):
    form = DropdownControlPanelForm



# -*- coding: utf-8 -*-
from plone import api
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

import logging


logger = logging.getLogger('webcouturier.dropdownmenu.upgrades')


def upgrade_1012_to_1030(context):
    """Upgrade to version 3.0"""

    # PORTAL PROPERTIES
    pprops = api.get_tool('portal_properties')
    if 'dropdown_properties' in pprops:
        pprops.manage_delObjects(['dropdown_properties'])  # noqa: P001
        logger.info(u'dropdown_properties removed from portal_properties')

    # PORTAL SKINS
    pskin = api.get_tool('portal_skins')
    for name, layers in pskin.getSkinPaths():
        layers = layers.split(',')
        if 'dropdownmenu_sunburst' in layers:
            layers.remove('dropdownmenu_sunburst')
            logger.info(
                u'skin layer dropdownmenu_sunburst removed from {0}'.format(
                    name
                )
            )
        if 'dropdownmenu' in layers:
            layers.remove('dropdownmenu')
            logger.info(
                u'skin layer dropdownmenu removed from {0}'.format(name)
            )
        pskin._getSelections()[name] = ','.join(layers)

    if 'dropdownmenu' in pskin:
        pskin.manage_delObjects(['dropdownmenu'])  # noqa: P001
        logger.info(u'skin path dropdownmenu removed')
    if 'dropdownmenu_sunburst' in pskin:
        pskin.manage_delObjects(['dropdownmenu_sunburst'])  # noqa: P001
        logger.info(u'skin path dropdownmenu_sunburst removed')

    # REGISTRY
    # pre-release cleanup
    registry = getUtility(IRegistry)
    regkey_enable_thumbs = (
        'webcouturier.dropdownmenu.browser.interfaces.'
        'IDropdownConfiguration.enable_thumbs'
    )
    if regkey_enable_thumbs in registry.records:
        del registry.records[regkey_enable_thumbs]
        logger.info(u'registry entry {0} removed'.format(regkey_enable_thumbs))

    # IMPORT REGISTRY PROFILE
    profile_id = 'profile-webcouturier.dropdownmenu:default'
    step_id = 'plone.app.registry'
    setup = api.get_tool('portal_setup')
    setup.runImportStepFromProfile(profile_id=profile_id, step_id=step_id)
    logger.info(
        u'profile {0} with step {1} imported'.format(profile_id, step_id)
    )
